@@ -1,6 +1,8 @@
 #undef PACKAGE
 #undef VERSION
+#undef AT_BINDIR
 #undef GETTEXT_PACKAGE
+#undef GDM_CONFIGDIR
 #undef GDM_USER_PATH
 #undef HALT_COMMAND
 #undef REBOOT_COMMAND
@@ -39,7 +41,6 @@
 #undef EXPANDED_AUTHDIR
 #undef EXPANDED_DATADIR
 #undef EXPANDED_BINDIR
-#undef EXPANDED_GDMCONFIGDIR
 #undef EXPANDED_LIBEXECDIR
 #undef EXPANDED_LIBDIR
 #undef EXPANDED_LOCALEDIR
@@ -48,9 +49,11 @@
 #undef EXPANDED_SBINDIR
 #undef EXPANDED_SYSCONFDIR
 #undef EXPANDED_SESSDIR
-#undef X_PATH
 #undef X_SERVER
 #undef X_SERVER_PATH
 #undef X_XNEST_PATH
 #undef X_XNEST_CONFIG_OPTIONS
 #undef X_CONFIG_OPTIONS
+#undef X_PATH
+#undef XSESSION_SHELL
+#undef XEVIE_OPTION
